#!/bin/ksh

i_am=trace.ksh

scale_default=50
x_default=300
y_default=400
op_default=fill

usage="
$i_am LEVEL [SCALE] [x] [y] [stroke|fill]

defaults:

    SCALE : $scale_default
        X : $x_default
        Y : $y_default
       OP : $scale_default
"
 
level=$1
scale=$2
x=$3
y=$4
op=$5
 
if [[ -z $level ]]
then
    echo "$usage"
    exit 1
fi

[[ -z $scale ]] && scale=$scale_default
[[ -z $x     ]] && x=$x_default
[[ -z $y     ]] && y=$y_default
[[ -z $op    ]] && op=$op_default

echo \
"0 0
1 0
0 1
-1 0
0 -1" |
 
awk \
    -v level=$level \
    -v scale=$scale \
    -v x=$x \
    -v y=$y \
    -v op=$op \
'BEGIN {
    h=0;
    SUBSEP = ",";
}

{
    x_disp[h]=$1;
    y_disp[h]=$2;
    ++h;
}

function recurse(height,x,y,h,x_base,y_base) {
    if (height == 0)
    {
        ind = x SUBSEP y SUBSEP x+1 SUBSEP y;
        if (ind in edge) { delete edge[ind]; } else
                         { edge[ind] = 1; }

        ind = x SUBSEP y SUBSEP x SUBSEP y+1;
        if (ind in edge) { delete edge[ind]; } else
                         { edge[ind] = 1; }

        ind = x SUBSEP y+1 SUBSEP x+1 SUBSEP y+1;
        if (ind in edge) { delete edge[ind]; } else
                         { edge[ind] = 1; }

        ind = x+1 SUBSEP y SUBSEP x+1 SUBSEP y+1;
        if (ind in edge) { delete edge[ind]; } else
                         { edge[ind] = 1; }
    }
    else
    {
        x_base = 2*x -   y;
        y_base =   x + 2*y;
        for (h=0; h<5; ++h)
            recurse(height-1, x_base+x_disp[h], y_base+y_disp[h]);
    }
}

function trace() {
    for (e in edge) {
        break;
    }

    print "%!PS";
    print "newpath", x, y, "moveto";

    split(e, a, /,/);
    x_old = a[1];
    y_old = a[2];
    x = a[3];
    y = a[4];

    while (1)
    {
        printf("%s %s rlineto\n", (x-x_old)*scale, (y-y_old)*scale);

        x_old = x;
        y_old = y;

        delete edge[e];
        e = "empty";

        x_new = x;
        y_new = y;

        ind = x SUBSEP y SUBSEP x+1 SUBSEP y;
        if ( ind in edge) { e = ind; x_new = x+1; }

        ind = x SUBSEP y SUBSEP x SUBSEP y+1;
        if ( ind in edge) { e = ind; y_new = y+1; }

        ind = x-1 SUBSEP y SUBSEP x SUBSEP y;
        if ( ind in edge) { e = ind; x_new = x-1; }

        ind = x SUBSEP y-1 SUBSEP x SUBSEP y;
        if ( ind in edge) { e = ind; y_new = y-1; }

        if (e ~ /empty/)
        {
            if (op ~ /stroke/) print "stroke";
            if (op ~ /fill/  ) print "fill";
            exit(0);
        } else {
            x = x_new;
            y = y_new;
        }
    }
}

END {
    recurse(level,0,0);
    trace();
}'
